// gameport.c - partial listing

//-----------------------------------------------------------------
NTSTATUS GamePortCreate(IN PDEVICE_OBJECT DevObject, IN PIRP Irp)
{
    NTSTATUS status = STATUS_SUCCESS;
    PIO_STACK_LOCATION irpStack = IoGetCurrentIrpStackLocation(Irp);

    KdPrint(("GamePortCreate\n"));

    if (irpStack->FileObject->FileName.Length != 0) {
        status = STATUS_INVALID_PARAMETER;
    }

    Irp->IoStatus.Status = status;
    Irp->IoStatus.Information = 0;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return status;

} // GamePortCreate

//-----------------------------------------------------------------
NTSTATUS GamePortClose(IN PDEVICE_OBJECT DevObject, IN PIRP Irp)
{
    KdPrint(("GamePortClose\n"));

    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return STATUS_SUCCESS;

} // GamePortClose

//-----------------------------------------------------------------
NTSTATUS GamePortRead(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp)
{
    NTSTATUS status = STATUS_SUCCESS;
    PIO_STACK_LOCATION irpStack = IoGetCurrentIrpStackLocation(Irp);
    PDEVICE_EXTENSION deviceExt = DeviceObject->DeviceExtension;

    KdPrint(("GamePortRead\n"));

    //
    // Validate parameters and read a byte from the gameport.
    //   

    if (irpStack->Parameters.Read.Length == 0) {
        status = STATUS_INVALID_PARAMETER;
        Irp->IoStatus.Information = 0;
    } else {
        *(PUCHAR)(Irp->AssociatedIrp.SystemBuffer) = 
            READ_PORT_UCHAR((PUCHAR)(deviceExt->TranslatedPort));
        Irp->IoStatus.Information = 1;   // 1 byte transfered
    }

    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return status;

} // GamePortRead

//-----------------------------------------------------------------
NTSTATUS GamePortWrite(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp)
{
    NTSTATUS status = STATUS_SUCCESS;
    PIO_STACK_LOCATION irpStack = IoGetCurrentIrpStackLocation(Irp);
    PDEVICE_EXTENSION deviceExt = DeviceObject->DeviceExtension;

    KdPrint(("GamePortWrite\n"));

    //
    // Validate parameters and write a byte to the gameport.
    //   

    if (irpStack->Parameters.Write.Length == 0) {
        status = STATUS_INVALID_PARAMETER;
        Irp->IoStatus.Information = 0;
    } else {
        WRITE_PORT_UCHAR((PUCHAR)(deviceExt->TranslatedPort),
            *(PUCHAR)(Irp->AssociatedIrp.SystemBuffer));
        Irp->IoStatus.Information = 1;   // 1 byte transfered
    }

    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);
    return status;

} // GamePortWrite

